// Load and exchange library for board MB7707

#include <linux/types.h>

#ifndef __MB7707LOAD__H__
#define __MB7707LOAD__H__

#define PLOAD_MAJOR_VERSION 1
#define PLOAD_MINOR_VERSION 0

#ifdef __cplusplus
extern "C"
{
#endif // __cplusplus

	// 32 bit unsigned NM memory element type
	typedef __u32 PL_Word;

	// 32 bit unsigned NM address type
	typedef __u32 PL_Addr;

	// Board descriptor
	struct PL_Board;

	// Processor on board descriptor
	struct PL_Access;

	// Library functions return values
	// (all library functions return result code)
	enum RetValue
	{
		PL_OK         = 0, // OK
		PL_ERROR      = 1, // Error
		PL_TIMEOUT    = 2, // Timeout of wait for operation
		PL_FILE       = 3, // Can not find file for load
		PL_BADADDRESS = 4, // Bad address ranges
		PL_BUSY       = 5  // Board busy
	};

	// Flag of the user's program work end
	const PL_Word PROGRAM_FINISHED = 0x2;

	//------------------//
	// Common functions //
	//------------------//

	// Return version of library in variables pointed by 'version_major' and
	// 'version_minor'
	int PL_GetVersion(int *version_major, int *version_minor);

	// Create descriptor for board number 'index'
	// Return board descriptor in variable pointed by 'board'.
	// Boards index count from 0.
	int PL_GetBoardDesc(const char *iname, PL_Board **board);

	// Close board descriptor and free descriptor memory
	int PL_CloseBoardDesc(PL_Board *board);

	//-----------------//
	// Board functions //
	//-----------------//

	// Send RESET signal to board and all processors on board
	int PL_ResetBoard(PL_Board *board);

/*	// Get mask of processors on board
	unsigned int PL_GetProcMask(PL_Board *board);

	// Set mask of processors on board
	unsigned int PL_SetProcMask(PL_Board *board, unsigned int mask);*/

	// Create descriptor for processor number 'procNo' on board
	// Return processor descriptor in variable pointed by 'access'.
	// Processor number is 0.
	int PL_GetAccess(
		PL_Board *board,
		unsigned int procNo,
		PL_Access **access);

	// Close processor descriptor
	int PL_CloseAccess(PL_Access *access);

/*	// Enable / disable interrupt from processors
	// board - board descriptor
	// mask - bitwise processor mask (0 - disable, 1 - enable)
	// prev - previous bitwise mask (returned)
	int PL_MaskInterrupt(PL_Board *board, PL_Word mask, PL_Word *prev);

	// Wait for interrupt from board
	// board - board descriptor
	// timeOut - time-out interval, in milliseconds
	//	If timeOut is zero, function returns immediately.
	//	If timeOut is INFINITE (0xFFFFFFFF), the function's time-out interval never elapses.
	// ProcMask - interrupt source processors (bitwise mask, returned)
	int PL_WaitInterrupt(PL_Board *board, PL_Word timeOut, PL_Word *ProcMask);

	// Cancel all waits for interrupt from board
	// board - board descriptor
	int PL_CancelWaitInterrupt(PL_Board *board);*/

	//---------------------//
	// Processor functions //
	//---------------------//

	// Load init code on core
	int PL_LoadInitCode(PL_Access *access);

	// Load user program on processor and start execution
	int PL_LoadProgramFile(PL_Access *access, const char *filename);

	// Read array from shared memory
	// block   - Pointer to destination buffer in PC memory
	// len     - Size of array in 32-bit words
	// address - Address of source array in NM memory
	int PL_ReadMemBlock(
		PL_Access *access,
		PL_Word *block,
		PL_Addr address,
		PL_Word len);

	// Write array in shared memory
	// block   - Pointer to source array in PC memory
	// len     - Size of array in 32-bit words
	// address - Address of destination array in NM memory
	int PL_WriteMemBlock(
		PL_Access *access,
		PL_Word *block,
		PL_Addr address,
		PL_Word len);

	// Barrier syncronization with program on board processor
	// value       - value sent to processor
	// returnValue - value received from processor
	int PL_Sync(PL_Access *access, PL_Word value, PL_Word *returnValue);

	// Barrier syncronization with program on board processor
	// Additionally perform two values exchange
	// value       - value sent to processor
	// outAddress  - address sent to processor
	// outLen      - size sent to processor
	// returnValue - value received from processor
	// inAddress   - address received from processor
	// inLen       - size received from processor
	// Values return only if pointers are not NULL
	int PL_SyncArray(
		PL_Access *access,    // Processor descriptor

		PL_Word value,        // Value sent to processor
		PL_Addr outAddress,   // Address sent to processor
		PL_Word outLen,       // Size sent to processor

		PL_Word *returnValue, // Value received from processor
		PL_Addr *inAddress,   // Address received from processor
		PL_Word *inLen);      // Size received from processor

	// Set timeout in milliseconds for waiting function
	// Used in all PL_SyncXXX functions.
	int PL_SetTimeout(unsigned int timeout);

/*	// Send interrupt on processor
	int PL_Interrupt(PL_Access *access);*/

	// Return status bits in variable 'status'
	int PL_GetStatus(PL_Access *access, PL_Word *status);

	// Getting user program return value
	int PL_GetResult(PL_Access *access, PL_Word *result);

	//----------------------------------------//
	// Special functions only in MB7707 board //
	//----------------------------------------//

	// On | off first LED on the board
	int PL_FirstLightOn(PL_Board *board);
	int PL_FirstLightOff(PL_Board *board);

	// On | off second LED on the board
	int PL_SecondLightOn(PL_Board *board);
	int PL_SecondLightOff(PL_Board *board);

#ifdef __cplusplus
} // extern "C"
#endif // __cplusplus

#endif // __MB7707LOAD__H__
